@@ -16,8 +16,7 @@ import itertools
 import json
 import math
 import re
-from copy import deepcopy
-from typing import Any, Dict, List, NoReturn, Optional, Tuple, Union
+from typing import Any, Dict, List, Optional, Tuple, Union
 
 import jsonschema
 from hypothesis.errors import InvalidArgument
@@ -243,12 +242,12 @@ def canonicalish(schema: JSONType) -> Dict[str, Any]:
     then = schema.pop("then", schema)
     else_ = schema.pop("else", schema)
     if if_ is not None and (then is not schema or else_ is not schema):
-        schema = {
-            "anyOf": [
+        if then not in (if_, TRUTHY) or else_ != TRUTHY:
+            alternatives = [
                 {"allOf": [if_, then, schema]},
                 {"allOf": [{"not": if_}, else_, schema]},
             ]
-        }
+            schema = canonicalish({"anyOf": alternatives})
     assert isinstance(schema, dict)
     # Recurse into the value of each keyword with a schema (or list of them) as a value
     for key in SCHEMA_KEYS:
@@ -511,6 +510,23 @@ def canonicalish(schema: JSONType) -> Dict[str, Any]:
             return FALSEY
         if len(schema) == len(schema["anyOf"]) == 1:
             return schema["anyOf"][0]  # type: ignore
+        types = []
+        # Turn
+        #   {"anyOf": [{"type": "string"}, {"type": "null"}]}
+        # into
+        #   {"type": ["string", "null"]}
+        for subschema in schema["anyOf"]:
+            if "type" in subschema and len(subschema) == 1:
+                types.extend(get_type(subschema))
+            else:
+                break
+        else:
+            # All subschemas have only the "type" keyword, then we merge all types
+            # into the parent schema
+            del schema["anyOf"]
+            new_types = canonicalish({"type": types})
+            schema = merged([schema, new_types])
+            assert isinstance(schema, dict)  # merging was certainly valid
     if "allOf" in schema:
         schema["allOf"] = [
             json.loads(enc)
@@ -552,69 +568,6 @@ TRUTHY = canonicalish(True)
 FALSEY = canonicalish(False)
 
 
-class LocalResolver(jsonschema.RefResolver):
-    def resolve_remote(self, uri: str) -> NoReturn:
-        raise HypothesisRefResolutionError(
-            f"hypothesis-jsonschema does not fetch remote references (uri={uri!r})"
-        )
-
-
-def resolve_all_refs(
-    schema: Union[bool, Schema], *, resolver: LocalResolver = None
-) -> Schema:
-    """
-    Resolve all references in the given schema.
-
-    This handles nested definitions, but not recursive definitions.
-    The latter require special handling to convert to strategies and are much
-    less common, so we just ignore them (and error out) for now.
-    """
-    if isinstance(schema, bool):
-        return canonicalish(schema)
-    assert isinstance(schema, dict), schema
-    if resolver is None:
-        resolver = LocalResolver.from_schema(deepcopy(schema))
-    if not isinstance(resolver, jsonschema.RefResolver):
-        raise InvalidArgument(
-            f"resolver={resolver} (type {type(resolver).__name__}) is not a RefResolver"
-        )
-
-    if "$ref" in schema:
-        s = dict(schema)
-        ref = s.pop("$ref")
-        with resolver.resolving(ref) as got:
-            if s == {}:
-                return resolve_all_refs(got, resolver=resolver)
-            m = merged([s, got])
-            if m is None:  # pragma: no cover
-                msg = f"$ref:{ref!r} had incompatible base schema {s!r}"
-                raise HypothesisRefResolutionError(msg)
-            return resolve_all_refs(m, resolver=resolver)
-    assert "$ref" not in schema
-
-    for key in SCHEMA_KEYS:
-        val = schema.get(key, False)
-        if isinstance(val, list):
-            schema[key] = [
-                resolve_all_refs(v, resolver=resolver) if isinstance(v, dict) else v
-                for v in val
-            ]
-        elif isinstance(val, dict):
-            schema[key] = resolve_all_refs(val, resolver=resolver)
-        else:
-            assert isinstance(val, bool)
-    for key in SCHEMA_OBJECT_KEYS:  # values are keys-to-schema-dicts, not schemas
-        if key in schema:
-            subschema = schema[key]
-            assert isinstance(subschema, dict)
-            schema[key] = {
-                k: resolve_all_refs(v, resolver=resolver) if isinstance(v, dict) else v
-                for k, v in subschema.items()
-            }
-    assert isinstance(schema, dict)
-    return schema
-
-
 def merged(schemas: List[Any]) -> Optional[Schema]:
     """Merge *n* schemas into a single schema, or None if result is invalid.
 
@@ -841,6 +794,10 @@ def merged(schemas: List[Any]) -> Optional[Schema]:
             elif out[k] != v and k in ALL_KEYWORDS:
                 # If non-validation keys like `title` or `description` don't match,
                 # that doesn't really matter and we'll just go with first we saw.
+                #
+                # TODO: note that this is NOT TRUE in the case of recursive references,
+                # where we might change the value at some location that a reference
+                # still points to!
                 return None
         out = canonicalish(out)
         if out == FALSEY:
